/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.stimulation;

import java.util.Vector;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import re.jpayet.mentdb.core.brain.Brain;
import re.jpayet.mentdb.core.command.Command;
import re.jpayet.mentdb.core.command.MQLValue;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.link.LinkManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.fx.DateFx;
import re.jpayet.mentdb.core.search.BasicSearchEngine;
import re.jpayet.mentdb.core.search.DeepSearchEngine;
import re.jpayet.mentdb.core.session.SessionThread;
import re.jpayet.mentdb.core.session.env.EnvManager;
import re.jpayet.mentdb.core.user.UserManager;
import re.jpayet.mentdb.tools.Misc;

public class StimulationManager {
	
	public static Vector<StimulationManager> userStimulationStack = new Vector<StimulationManager>();
	
	//Properties
	public String fromUser = "", toUser = "", sentence = "";
	public long timestamp = 0;
	public Vector<MQLValue> inputVector = new Vector<MQLValue>();
	
	//Constructor
	public StimulationManager(long timestamp, String fromUser, String toUser, Vector<MQLValue> inputVector, String sentence) {
		this.timestamp=timestamp;
		this.fromUser=fromUser;
		this.toUser=toUser;
		this.inputVector=inputVector;
		this.sentence=sentence;
	}
	
	//Add the stimulation into the user stack
	public static void addIntoTheUserStack(long timestamp, String fromUser, String toUser, Vector<MQLValue> inputVector, String sentence) throws Exception {

		//Create the stimulation object
		StimulationManager stm = new StimulationManager(timestamp, fromUser, toUser, inputVector, sentence);
		
		//Add into the stack
		userStimulationStack.addElement(stm);
		
	}
	
	//Show stimulation
	public static String show(JSONArray stimulation) throws Exception {
		
		return stimulation.toJSONString();
				
	}
	
	//Get the size of the stimulation
	public static String size(JSONArray stimulation) throws Exception {
		
		return stimulation.size()+"";
				
	}
	
	//Get a stimulation
	public static JSONArray get(int i, JSONArray stimulation) throws Exception {
		
		if (stimulation.size()==0) {
			
			throw new Exception("Sorry, there is no stimulation.");
			
		}
		
		if (i<0) {
			
			throw new Exception("Sorry, the index must be >= 0.");
			
		}

		if (i>=stimulation.size()) {
			
			throw new Exception("Sorry, the index must be < "+stimulation.size()+".");
			
		}
		
		try {
			
			return (JSONArray) stimulation.get(i);
			
		} catch (Exception e) {
			
			return new JSONArray();
			
		}
				
	}
	
	//Set a stimulation
	@SuppressWarnings("unchecked")
	public static void set(String fromUser, String json) throws Exception {
		
		JSONParser jsonParser = new JSONParser();
		JSONArray a = (JSONArray) jsonParser.parse(json);
		Brain.stimulation.put(fromUser, a);
				
	}
	
	//create tab
	@SuppressWarnings("unchecked")
	public static void createTab(String fromUser) throws Exception {
		
		if (!Brain.stimulation.containsKey(fromUser)) {
			Brain.stimulation.put(fromUser, new JSONArray());
		}
				
	}
	
	//Remove a stimulation
	public static void remove(int i, JSONArray stimulation) throws Exception {
		
		if (stimulation.size()==0) {
			
			throw new Exception("Sorry, there is no stimulation.");
			
		}
		
		if (i<0) {
			
			throw new Exception("Sorry, the index must be >= 0.");
			
		}

		if (i>=stimulation.size()) {
			
			throw new Exception("Sorry, the index must be < "+stimulation.size()+".");
			
		}
		
		stimulation.remove(i);
				
	}
	
	@SuppressWarnings("unchecked")
	public static synchronized String call(boolean initialization, String fromUser, String str, EnvManager env, SessionThread session) throws Exception {
		
		if (!Brain.stimulation.containsKey(fromUser)) {
			Brain.stimulation.put(fromUser, new JSONArray());
		}
		
		if (initialization) {
			
			return Command.executeAllCommands(false, session, Misc.splitCommand("execute \"process<initialize>\" \"[fromUser]\" \""+fromUser.replace("\"", "\\\"")+"\" \"[str]\" \""+str.replace("\"", "\\\"")+"\";"), env);
			
		} else {
			
			return Command.executeAllCommands(false, session, Misc.splitCommand("execute \"process<click>\" \"[fromUser]\" \""+fromUser.replace("\"", "\\\"")+"\";"), env);
			
		}
		
	}
	
	public static JSONArray basic_search(JSONArray array, String typeList) throws Exception {
		
		return BasicSearchEngine.execute("ai", ((JSONArray) ((JSONObject) array.get(0)).get("reader_found_word")), typeList);
		
	}
	
	public static String search_strategy(JSONArray array) throws Exception {
		
		//Initialization
		Integer i_basic_search = ((Double) ((JSONObject) array.get(0)).get("i_basic_search")).intValue();
		JSONArray relations = (JSONArray) ((JSONObject) array.get(i_basic_search)).get("ordered_basic_search");
		JSONObject exclusion = (JSONObject) ((JSONObject) array.get(i_basic_search)).get("exclusion");
		int concentration = ConcentrationManager.getConcentrationDepth("C[deep-search]");
		
		//Search a logical relation
		JSONArray result = DeepSearchEngine.execute_from_brain(relations, "AS", "ASLGIOUDR", 1, concentration, exclusion);
		
		if (result.size()==0) {
			return null;
		} else {
			return Misc.atom(result.get(0)+"", 1, " ");
		}
		
	}
	
	public static String search_logical(JSONArray array) throws Exception {
		
		//Initialization
		Integer i_basic_search = ((Double) ((JSONObject) array.get(0)).get("i_basic_search")).intValue();
		JSONArray relations = (JSONArray) ((JSONObject) array.get(i_basic_search)).get("ordered_basic_search");
		JSONObject exclusion = (JSONObject) ((JSONObject) array.get(i_basic_search)).get("exclusion");
		int concentration = ConcentrationManager.getConcentrationDepth("C[deep-search]");
		
		//Search a logical relation
		JSONArray result = DeepSearchEngine.execute_from_brain(relations, "L", "ASLGIOUDR", 1, concentration, exclusion);
		
		if (result.size()==0) {
			return null;
		} else {
			return Misc.atom(result.get(0)+"", 1, " ");
		}
		
	}
	
	//redirection
	@SuppressWarnings("unchecked")
	public static void redirect(String data, JSONArray stimulation) throws Exception {
		
		//Checker
		if (stimulation.size()==0) {
			
			throw new Exception("Sorry, you must initialize the stimulation tab before.");
			
		}
		
		JSONArray array = ((JSONArray) stimulation.get(stimulation.size()-1));

		JSONObject stm = new JSONObject();
		stm.put("_current_action_", "REDIRECTION");
		stm.put("_next_action_", "redirection_relation");
		stm.put("redirection_data", data);
		
		array.add(stm);
				
	}
	
	public static void aiSaySomething(String sentence, SessionThread session) throws Exception {
		
		//Save the relation
		String thoughtList = "";
		String[] words = sentence.split(" ");
		for(int i=0;i<words.length;i++) {

			//Create thought
			String th = WordManager.create(session.user, words[i], "", "io");
			thoughtList += " "+th;

		}
		if (thoughtList.length()>0) thoughtList = thoughtList.substring(1);

		String curR_I = RelationManager.create(session.user, thoughtList, "io", "O");
		String[] time = DateFx.systimestamp(" ");
		String year = WordManager.create(session.user, time[0], "", "fr");
		String month = WordManager.create(session.user, time[1], "", "fr");
		String day = WordManager.create(session.user, time[2], "", "fr");
		String hour = WordManager.create(session.user, time[3], "", "fr");
		String minute = WordManager.create(session.user, time[4], "", "fr");
		String second = WordManager.create(session.user, time[5], "", "fr");
		RelationManager.createSubWithPosition(session.user, LinkManager.get_value("@[r_fr_quelquun_a_dit]"), 
				UserManager.getThought(session.user)+" "+LinkManager.get_value("@[th_fr_a]")+" "+LinkManager.get_value("@[th_fr_dit]")+" "+curR_I+" "+LinkManager.get_value("@[th_fr_le]")+" "+day+" "+month+" "+year+" "+LinkManager.get_value("@[th_fr_à]")+" "+hour+" "+LinkManager.get_value("@[th_fr_heure]")+" "+minute+" "+LinkManager.get_value("@[th_fr_minute]")+" "+LinkManager.get_value("@[th_fr_et]")+" "+second+" "+LinkManager.get_value("@[th_fr_seconde]"), 
				"1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16", 
				"io", "O");
		
	}

}
