/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.parameter;

import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;

public class ParameterManager {
	
	public static JSONObject allParameters = null;
	
	//Create the parameter object
	public static void init(String user) throws Exception {
		
		JSONObject param = new JSONObject();
		
		Record.add(user, "P[]", param.toJSONString());

		add(user, "ELEVATION_MAX_%_ACCEPTANCE", "75", "1");
		
	}
	
	//Load the parameter object
	public static void load() throws Exception {
		
		allParameters = Record.getNode("P[]");
		
	}

	//Generate update
	public static String generateUpdate(String param) throws Exception {
		
		//Generate an error if the parameter does not exist
		if (!exist(param)) {

			throw new Exception("Sorry, the parameter "+param+" does not exist.");

		}
		
		//Get the JOB object
		JSONObject p = Record.getNode("P[]");
		
		String result = "parameter update \""+param.replace("\"", "\\\"")+"\" \""+(""+((JSONObject) p.get(param)).get("value")).replace("\"", "\\\"")+"\";";
		
		return result;
		
	}
	
	public static JSONObject showAllParams(String user) throws Exception {
		
		return allParameters;
		
	}
	
	public static String get_value(String user, String key) throws Exception {
		
		
		if (!allParameters.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}
		
		return ((JSONObject) allParameters.get(key)).get("value")+"";
		
	}
	
	public static String get_looked(String user, String key) throws Exception {
		
		if (!allParameters.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}
		
		return ((JSONObject) allParameters.get(key)).get("looked")+"";
		
	}
	
	public static boolean exist(String key) throws Exception {
		
		return allParameters.containsKey(key);
		
	}
	
	@SuppressWarnings("unchecked")
	public static String add(String user, String key, String val, String looked) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("P[]");
		
		if (recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" already exist.");
			
		}
		
		JSONObject value = new JSONObject();
		value.put("value", val);
		value.put("looked", looked);

		recNode.put(key, value);
		
		Record.update(user, "P[]", recNode.toJSONString());
		
		allParameters = recNode;
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static String update(String user, String key, String val) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("P[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}
		
		if ((((JSONObject) recNode.get(key)).get("looked")+"").equals("1")) {
			
			throw new Exception("Sorry, the parameter "+key+" is looked.");
			
		}

		((JSONObject) recNode.get(key)).put("value", val);
		
		Record.update(user, "P[]", recNode.toJSONString());
		
		allParameters = recNode;
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static String look(String user, String key) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("P[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}

		((JSONObject) recNode.get(key)).put("looked", "1");
		
		Record.update(user, "P[]", recNode.toJSONString());
		
		allParameters = recNode;
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static String unlook(String user, String key) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("P[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}

		((JSONObject) recNode.get(key)).put("looked", "0");
		
		Record.update(user, "P[]", recNode.toJSONString());
		
		allParameters = recNode;
		
		return result;
		
	}
	
	public static String remove(String user, String key) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("P[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the parameter "+key+" does not exist.");
			
		}
		
		if ((((JSONObject) recNode.get(key)).get("looked")+"").equals("1")) {
			
			throw new Exception("Sorry, the parameter "+key+" is looked.");
			
		}

		recNode.remove(key);
		
		Record.update(user, "P[]", recNode.toJSONString());
		
		allParameters = recNode;
		
		return result;
		
	}

}
