/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.sequence;

import java.math.BigInteger;

import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;

public class SequenceManager {
	
	//Create the sequence object
	@SuppressWarnings("unchecked")
	public static void init(String user) throws Exception {
		
		JSONObject seq = new JSONObject();
		seq.put("thought", "-1");
		seq.put("circle", "-1");
		seq.put("relation-G", "-1"); //Group > OK
		seq.put("relation-L", "-1"); //Logic (always true) > OK
		seq.put("relation-I", "-1"); //Input > OK
		seq.put("relation-O", "-1"); //Output > OK
		seq.put("relation-U", "-1"); //Usage > OK Simple usage relation
		seq.put("relation-S", "-1"); //Silent Strategy > OK
		seq.put("relation-A", "-1"); //Action Strategy > OK
		seq.put("relation-R", "-1"); //Response > OK
		seq.put("relation-D", "-1"); //Directional > OK
		
		Record.add(user, "SQ[]", seq.toJSONString());
		
	}
	
	@SuppressWarnings("unchecked")
	public static String incr(String user, String key) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		//Load the node
		BigInteger seq = new BigInteger(""+recNode.get(key), 36);
		BigInteger one = new BigInteger("1");
		result = seq.add(one).toString(36);
		recNode.put(key, result);
		
		Record.update(user, "SQ[]", recNode.toJSONString());
		
		return result;
		
	}
	
	public static String showAllSeqs() throws Exception {
		
		return Record.getNode("SQ[]").toJSONString();
		
	}

	//Generate update
	public static String generateUpdate(String key) throws Exception {
		
		//Generate an error if the sequence does not exist
		if (!exist(key)) {

			throw new Exception("Sorry, the sequence "+key+" does not exist.");

		}
		
		JSONObject s = Record.getNode("SQ[]");
		
		String result = "sequence update \""+key+"\" \""+s.get(key)+"\";";
		
		return result;
		
	}
	
	public static String get_current(String user, String key) throws Exception {
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the sequence id "+key+" does not exist.");
			
		}
		
		return recNode.get(key)+"";
		
	}
	
	public static boolean exist(String key) throws Exception {
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		return recNode.containsKey(key);
		
	}
	
	@SuppressWarnings("unchecked")
	public static String add(String user, String key, String val) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		if (recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the sequence id "+key+" already exist.");
			
		}
		
		recNode.put(key, val);
		
		Record.update(user, "SQ[]", recNode.toJSONString());
		
		return result;
		
	}
	
	@SuppressWarnings("unchecked")
	public static String update(String user, String key, String val) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the sequence id "+key+" does not exist.");
			
		}
		
		recNode.put(key, val);
		
		Record.update(user, "SQ[]", recNode.toJSONString());
		
		return result;
		
	}
	
	public static String remove(String user, String key) throws Exception {
		
		//Initialization
		String result = "";
		
		//Get the node
		JSONObject recNode = Record.getNode("SQ[]");
		
		if (!recNode.containsKey(key)) {
			
			throw new Exception("Sorry, the sequence id "+key+" does not exist.");
			
		}
		
		recNode.remove(key);
		
		Record.update(user, "SQ[]", recNode.toJSONString());
		
		return result;
		
	}

}
